/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


////////////////////////////////////////////////////////////////////////////
//
// Document Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <string.h>

#include "OO7.hxx"
#include "GenParams.h"
#include "VarParams.h"

extern int	debugMode;

////////////////////////////////////////////////////////////////////////////
//
// Document Constructor
//
////////////////////////////////////////////////////////////////////////////

Document::Document(int cpId) : id(cpId)
{
    // prepare and fill in the (indexed) document title

    char docTitle[TitleSize];
    sprintf(docTitle, "Composite Part %08d", cpId);
    if (debugMode) {
        printf("Document::Document(title = %s)\n", docTitle);
    }
    
    title = docTitle ;

    // prepare and fill in the document text

    char myText[DocTextLength];
    sprintf(myText, DocumentText, cpId);
    int stringLen = strlen(myText);
    
    text.alloc(DocumentSize) ;
    int curChar, strChar;
    curChar = strChar = 0;
    while (curChar < DocumentSize-1) {
	text[curChar++] = myText[strChar++];
	if (strChar >= stringLen) { strChar = 0; }
    }
    text[curChar] = '\0';

    // add document to Document extent
    Document::extent->insert(this) ;
}


////////////////////////////////////////////////////////////////////////////
//
// Document searchText Method for use in traversals
//
////////////////////////////////////////////////////////////////////////////

int Document::searchText(char c)
{
    if (debugMode) {
        printf("                    Document::searchText(title = %s)\n", (char*)title);
    }

    // count occurrences of the indicated letter (for busy work)

    int i = 0;
    int count = 0;
    char* mytext = this->text ;
    while (mytext[i] != '\0') {
	if (mytext[i++] == c) { count++; }
    }

    if (debugMode) {
        printf("                    [found %d '%c's among %d characters]\n",
	        		     count, c, i);
    }

    return count;
}


////////////////////////////////////////////////////////////////////////////
//
// Document replaceText Method for use in traversals
//
////////////////////////////////////////////////////////////////////////////

int Document::replaceText(char *oldString, char *newString)
{
    if (debugMode) {
        printf("                    Document::changeText(title = %s)\n", (char*)title);
    }

    // check to see if the text starts with the old string

    int oldTextSize = ::strlen(text) + 1;
    int oldStrLength  = ::strlen(oldString);
    int foundMatch = (strncmp(text, oldString, oldStrLength) == 0);

    // if so, change it to start with the new string instead

    if (foundMatch) {

        int newStrLength  = ::strlen(newString);
        int lengthDiff = newStrLength - oldStrLength;
        int newTextSize  = oldTextSize + lengthDiff;

	if (lengthDiff == 0) {
	    strncpy(text, newString, newStrLength);
        } else {
	    char* newText = new char[newTextSize];
	    strncpy(newText, newString, newStrLength);
	    strncpy(&newText[newStrLength], &text[oldStrLength],
		    oldTextSize - oldStrLength);
            text = newText;
        }
    }

    if (debugMode) {
	if (foundMatch) {
            printf("                    [changed \"%s\" to \"%s\"]\n",
	                                 oldString, newString);
	} else {
            printf("                    [no match, so no change was made]\n");
	}
    }

    if (foundMatch) {
        return 1;
    } else {
        return 0;
    }

}


////////////////////////////////////////////////////////////////////////////
//
// Document Destructor
//
////////////////////////////////////////////////////////////////////////////

Document::~Document()
{
    if (debugMode) {
        printf("Document::~Document(title = %s)\n", (char*)title);
    }

    // remove document from Document extent
    Document::extent->remove(this) ;
}


